//--------------------------------------------------------------------
//      gen.h 
//      ----- 
//      Generator of the "sql" file with spatio-bitemporal data and queries
//      Defined: TGenerator
//
//      Spatio-temporal data generator v 1.0 
//      Copyright(c) 1998-2000, Aalborg University
//

#ifndef __GEN_H
#define __GEN_H

#include <fstream.h>
#include "params.h"
#include "pool.h"

extern bool gen_binary;

//-------------------------------------------------------------------
class TGenerator
{
 public:
   TGenerator (TParams& params);
   void Generate ();

 private:
   void ReadSpatial     (TSpBitemporal& key);
   void AdjustSpatial   (TSpBitemporal& to, TSpBitemporal& from);
   void StartHistory    (TPool& pool, long CT);
   void EndHistory      (TPool& pool, long CT);
   void UpdateHistory   (TPool& pool, long CT);
   void ContinueHistory (TPool& pool, long CT); 
   void GenerateQueries (long CT);

   char   table_name[30];
   char   spfile_name[30];
   bool   pointSpData;       // whether to interpret spatial data as points
   long   total;             // Total number of logical index operations
   long   initialInserts;    // Initial inserts
   long   beginning;         // Beginning Time
   double probStart;         // Probabilities of operations
   double probEnd;           // (at a granularity of object histories)
   double probUpdInsert;     // Probabilities of operations on a history  
   double probUpdDelete;     // if update is performed on that history  

                             // Parameters for insert
   double devVt1;            // Deviation of valid time interval beginning
   long   maxVtLength;       // Maximum valid time interval length
   double devDelta;          // Deviation of Delta offset
   long   spChange;          // Maximum delta x and y for changing spatial regions
   double updatesPerPoint;   // how many updates per one time point
                             // Parameters for querying 
   long   interval;          // interval when to generate queries
   long   queryQty;          // number of queries to generate each time
   double queryTempSpR;      // Tmporal x Spatial = 1 x QueryTempSpR - space-time ratio
   double queryVolume;       // Four-dimensional volume of a query
   long   queryType;         // 0-window, 1-spatial point, 2-temproal point, 3-tt slice
   long   maxQWindowTT;      // maximum query window transaction time extent
   long   maxQWindowVT;      // maximum query window valid time extent
   long   maxQWindowSp;      // maximum query window spatial extent
   double probBitempQ;       // probability of purely bitemporal query   
   double probSpatialQ;      // probability of purely spatial query
   double slices;            // probability of transaction time slice queiry
   double windows;           // probability of window-query
   double points;            // probability of point-query
   double spPoints;          // probability of spatial point query
   double current;           // probability of current-time query

   long   statsStart;        // time point when to print "stats out"
   long   statsInterval;     // time interval when to print "stats out"

   long   minX, minY;        // Extent of a spatial universe -
   long   maxX, maxY;        // read from the first line of spatial data file
   long   maxFutureVT;       // How far in the valid time beyond the current 
                             // transaction time objects extend
   unsigned short maxKey;    // basically, number of generated histories so far

   ofstream sqlf;     // generated "sql" file
   ifstream spdf;     // file with spatial data
};
#endif


