//--------------------------------------------------------------------
//      pool.cpp 
//      -------- 
//      Pool of the spatio-bitemporal data objects
//      Implemented: TSpBitemporal, THistory, TPool
//
//      Spatio-temporal data generator v 1.0 
//      Copyright(c) 1998-2000, Aalborg University
//

#include "pool.h"
//#define NDEBUG
#include <assert.h>
#include <string.h>
#include <limits.h>

//------------------------------------------------
//  THistory
//
THistory::~THistory()
{
   if (array) delete[] array;
}

//------------------------------------------------
void THistory::setLen (int n)
{
   if (n > len)
   {
      len = n;

      TSpBitemporal* temp = new TSpBitemporal[len];
      if (array)
      { 
         for (int i = 0; i < num; i++) temp[i] = array[i];
         delete[] array;
      }
      array = temp;
   }
}

//------------------------------------------------
TSpBitemporal& THistory::operator[] (int n) const
{
   assert(n < num);
   return array[n];
}

//------------------------------------------------
void THistory::Add (TSpBitemporal& elem, int n)
{
   if (n   == -1 ) n = num;
   if (num == len) setLen(len + HISTORY_INC);

   memmove(array + n + 1, array + n, (num - n)*sizeof(TSpBitemporal));
   array[n] = elem;
   num++;
}

//------------------------------------------------
void THistory::Delete (int n)
{
   assert(n < num && n >= 0);
   memmove(array + n, array + n + 1, (num - n - 1)*sizeof(TSpBitemporal));
   num--;
}

//------------------------------------------------
//  TPool 
//  TPool "ows" THistories, i.e., it deltes them in Delete and ~TPool
//
TPool::~TPool()
{
   if (array)
   {
      for (int i = 0; i < num; i++) delete array[i]; 
      delete[] array;
   }
}

//------------------------------------------------
void TPool::setLen (int n)
{
   if (array) delete[] array;

   array = new THistory*[n];
   len   = n;
   num   = 0; 
}

//------------------------------------------------
THistory* TPool::operator[] (int n) const
{
   assert(n < num);
   return array[n];
}

//------------------------------------------------
void TPool::Add (THistory* hist)
{
   if (num == len)
   {
      len += POOL_INC;

      THistory** temp = new THistory*[len];

      for (int i = 0; i < num; i++) temp[i] = array[i];
      delete[] array;
      array = temp;
   }
   array[num++] = hist;
}

//------------------------------------------------
void TPool::Delete (int n)
{
   assert(n < num && n >= 0);
   delete array[n];
   memmove (array + n, array + n + 1, (num - n - 1) * sizeof (THistory*));
}

//------------------------------------------------
void TPool::Schedule (int n)
{
   assert(n < num && n >= 0);

   int       i;
   THistory* hist = array[n];   
   
   if (hist->futureEntry.tt1 == LONG_MAX)     // Ended history
      i = numEnded++;
   else 
      for (i = n; i && array[i-1]->futureEntry.tt1 < hist->futureEntry.tt1; i--);
   memmove (array + i + 1, array + i, (n - i) * sizeof(THistory*));
   array[i] = hist;
}
