//--------------------------------------------------------------------
//      random.cpp 
//      ---------- 
//      Generation of normally distributed random numbers
//      Adopted from Tom Schneider
//
//      Spatio-temporal data generator v 1.0 
//      Copyright(c) 1998-2000, Aalborg University
//


#include "random.h"
#include <math.h>

/*
   Method: if U is a member of the set [0..1] and Un and Un+1
   are two members, then define
      theta = Un 2 pi
      r     = sqrt(-2 ln(Un+1))
   then when these polar coordinates are converted to Cartesian
   coordinates, one gets two independent Normally distributed numbers,
   with mean 0 and standard deviation 1.  To get other standard deviations
   multiply by a constant, and to get other means, add a constant.
*/

double randnorm (double mean, double deviation)
{
   double uniform;
   double theta, r;
   double x;

   do
   {
      theta = 2.0 * M_PI * rnd();
      uniform = rnd();
      // avoid bombing if the uniform happens to be zero
      if (uniform != 0.0)	r = sqrt(-2 * log(uniform));
   }
   while (uniform == 0.0);

   x = r * cos(theta);         // from polar to cartesian coordinates

   return mean + x*deviation;  // return x coordinate
}


